/* Stylesheet for the DJ Press default template. */

:root {
  --dark0: #2e3440;
  --dark1: #3b4252;
  --dark2: #434c5e;
  --dark3: #4c566a;
  --light4: #d8dee9;
  --light5: #e5e9f0;
  --light6: #eceff4;
  --primary7: #8fbcbb;
  --primary8: #88c0d0;
  --secondary9: #81a1c1;
  --tertiary10: #5e81ac;
  --error11: #bf616a;
  --danger12: #d08770;
  --warning13: #ebcb8b;
  --success14: #a3be8c;
  --info15: #b48ead;
}

body {
  font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Ubuntu,
               sans-serif;
  color: var(--dark0);
  margin: 0;
  padding: 0;
  box-sizing: border-box;
  background-color: var(--light5);
}

/* Top header */

body > header {
  background-color: var(--dark1);
  padding: 1rem calc((100% - 992px) / 2);
  display: flex;
  align-items: center;
  justify-content: space-between;
}

body > header h1 {
  margin: 0;
  font-size: 1.5rem;
}

body > header h1 a {
  color: var(--light6);
  text-decoration: none;
  padding: 0.5rem 1rem;
}

body > header ul {
  list-style: none;
  margin: 0;
  padding: 0;
  display: flex;
}

body > header ul li {
  margin-left: 1rem;
}

body > header ul li a {
  color: var(--light6);
  text-decoration: none;
  padding: 0.5rem 1rem;
  border-radius: 0.25rem;
  transition: background-color 0.2s;
}

body > header ul li a:hover {
  background-color: var(--dark3);
  color: var(--primary8);
}

/* Main section */

main {
  padding: 2em;
  max-width: 992px;
  margin: 0 auto 3em;
}

main > h1 {
  font-size: 2rem;
  margin: 0 0.5em 1em;
  color: var(--dark3);
}
/* Articles */

/* General article styles */
article {
  margin: 0 0 2em;
  padding: 2em;
  border: 1px solid var(--light4);
  border-radius: 0.5em;
  background-color: var(--light6);
  box-shadow: 0 2px 4px hsl(0deg 0% 0% / 0.1);
}

/* Article header styles */
article > header {
  margin-bottom: 1rem;
  border-bottom: 1px solid var(--light4);
  padding-bottom: 0.5rem;
}

article > header h1 {
  font-size: 1.75rem;
  margin: 0;
}

article > header h1 a {
  text-decoration: none;
  color: var(--tertiary10);
  transition: color 0.2s;
}

article > header h1 a:hover {
  color: var(--primary8);
}

article > header p {
  margin: 0.5em 0;
  font-size: 0.875rem;
  color: var(--dark3);
}

/* Article section styles */
article > section {
  margin-bottom: 1em;
}

/* Article footer styles */
article > footer {
  border-top: 1px solid var(--light4);
  padding-top: 0.5em;
  margin-top: 1em;
}

article > footer p {
  margin: 0;
  color: #555555;
}

/*noinspection CssUnusedSymbol*/
.badge {
  background-color: var(--primary8);
  color: var(--dark0);
  padding: 0.375em 0.5em;
  border-radius: 0.75em;
  font-size: 0.875rem;
  margin-right: 0.5em;
  font-weight: bold;
  text-decoration: none;
}

/*noinspection CssUnusedSymbol*/
.badge:hover {
  background-color: var(--tertiary10);
  color: var(--light4);
}

/* Footer section */

body > footer {
  background-color: var(--dark1);
  padding: 1rem calc((100% - 992px) / 2);
  display: flex;
  align-items: center;
  justify-content: space-between;
  position: fixed;
  bottom: 0;
  width: 100%;
  color: var(--light6);
}

body > footer p {
  font-size: 1rem;
  margin: 0;
}

body > footer a {
  color: var(--light6);
}
body > footer a:hover {
  color: var(--primary8);
}
