body {
  display: grid;
  grid-template-columns: 0.6fr 1.4fr;
  grid-template-rows: min-content 1.8fr min-content;
  gap: 0px 0px;
  grid-auto-flow: row;
  grid-template-areas:
  "Header Header"
  "Sidebar Content"
  "Footer Footer";
  width: 80%;
  height: 100%;
  margin: 0 auto;
}

header {
  grid-area: Header;
}

nav {
  grid-area: Sidebar;
}

main {
  grid-area: Content;
}

footer {
  grid-area: Footer;
}
