# Generated by Django 5.1.3 on 2024-11-20 11:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("djpress", "0006_alter_post_parent"),
    ]

    operations = [
        migrations.CreateModel(
            name="PluginStorage",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("plugin_name", models.CharField(max_length=100, unique=True)),
                ("plugin_data", models.JSONField(default=dict)),
            ],
            options={
                "verbose_name": "plugin storage",
                "verbose_name_plural": "plugin storage",
            },
        ),
    ]
