# Generated by Django 5.1.2 on 2024-10-09 06:36

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("djpress", "0005_post_parent"),
    ]

    operations = [
        migrations.AlterField(
            model_name="post",
            name="parent",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"post_type": "page"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="children",
                to="djpress.post",
            ),
        ),
    ]
