# Generated by Django 5.0.4 on 2024-05-03 11:04

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("djpress", "0001_initial"),
    ]

    operations = [
        migrations.RenameField(
            model_name="post",
            old_name="content_type",
            new_name="post_type",
        ),
    ]
