# Generated by Django 5.1.3 on 2024-11-27 23:23

import django.db.models.deletion
import django.db.models.manager
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("djpress", "0007_pluginstorage"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="post",
            options={"default_manager_name": "admin_objects", "verbose_name": "post", "verbose_name_plural": "posts"},
        ),
        migrations.AlterModelManagers(
            name="post",
            managers=[
                ("admin_objects", django.db.models.manager.Manager()),
            ],
        ),
        migrations.AlterField(
            model_name="post",
            name="categories",
            field=models.ManyToManyField(blank=True, related_name="_posts", to="djpress.category"),
        ),
        migrations.AlterField(
            model_name="post",
            name="parent",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"post_type": "page"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="_children",
                to="djpress.post",
            ),
        ),
    ]
